/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.portlet.sf;

import hu.midori.kosmos.model.SfRelease;
import hu.midori.kosmos.portlet.AbstractKosmosPortlet;
import hu.midori.kosmos.protocol.SfService;

import java.net.MalformedURLException;
import java.util.Date;
import java.util.List;

import javax.portlet.PortletConfig;
import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.caucho.hessian.client.HessianProxyFactory;

/**
 * Monitors <i>SourceForge</i> projects and releases.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class SfMonitoringPortlet extends AbstractKosmosPortlet {
	private final static Log log = LogFactory.getLog(SfMonitoringPortlet.class);

	private SfMonitoringConfig monitoringConfig = null;

	@Override
	public void init(PortletConfig config) throws PortletException {
		log.info("Initializing...");
		super.init(config);

		String monitoredResource = getMonitoredResourceInitParameter(config);
		String serviceUrl = getServiceUrlInitParameter(config);
		List<String> monitoredUrls = getMonitoredUrlsInitParameter(config);

		// save config
		monitoringConfig = new SfMonitoringConfig(monitoredResource, serviceUrl, monitoredUrls);
	}

	@Override
	public void destroy() {
		log.info("Destroying...");
	}

	@Override
	public void doView(RenderRequest request, RenderResponse response) {
		log.info("Viewing...");

		// connect to web-service
		SfService service = null;
		HessianProxyFactory factory = new HessianProxyFactory();
		try {
			service = (SfService)factory.create(SfService.class, monitoringConfig.getServiceUrl());
		} catch (MalformedURLException ex) {
			log.error("Unable to connect to the service", ex);
		}

		// get file-release list if the service was successfully set-up
		Date timestamp = null;
		List<SfRelease> releases = null;
		if(service != null) {
			for(String monitoredUrl : monitoringConfig.getMonitoredUrls()) {
				if(releases == null)
					releases = service.getFileReleases(monitoredUrl);
				else
					releases.addAll(service.getFileReleases(monitoredUrl));
			}
			log.debug("SF releases downloaded: " + releases.size());

			timestamp = service.getTimestamp();
			log.debug("SF timestamp: " + timestamp);
		}

		// render response
		getPortletContext().setAttribute("timestamp", timestamp);
		getPortletContext().setAttribute("monitoringConfig", monitoringConfig);
		getPortletContext().setAttribute("releases", releases);

		renderJsp(request, response, "/pages/sf_monitoring.jsp", String.format("%s - %d", monitoringConfig.getMonitoredResource(), releases.size()));
	}

	@Override
	public void doHelp(RenderRequest request, RenderResponse response) {
		log.info("Helping...");

		// render response
		renderJsp(request, response, "/pages/sf_monitoring_help.jsp", monitoringConfig.getMonitoredResource());
	}
}
